#ifndef BatchLab_AnalysisFactory_h
#define BatchLab_AnalysisFactory_h

#include <AIDA/IAnalysisFactory.h>

#include <AIDA/IConstants.h> //For AIDA_VERSION

namespace Slash {namespace Core {class ISession;}}
namespace Slash {namespace Core {class IWriter;}}
#include <Slash/Core/IManager.h>

namespace BatchLab {

// A factory manager.

class AnalysisFactory 
:public virtual Slash::Core::IManager
,public virtual AIDA::IAnalysisFactory {
public: //AIDA/IAnalysisFactory
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual AIDA::ITreeFactory* createTreeFactory();
  virtual AIDA::IHistogramFactory* createHistogramFactory(AIDA::ITree&);
  virtual AIDA::ITupleFactory* createTupleFactory(AIDA::ITree&);
  virtual AIDA::IFunctionFactory* createFunctionFactory(AIDA::ITree&);
  virtual AIDA::IPlotterFactory* createPlotterFactory(int,char**,
                                                      const std::string&,
                                                      const std::string&);
  virtual AIDA::IDataPointSetFactory* createDataPointSetFactory(AIDA::ITree&);
  virtual AIDA::IFitFactory* createFitFactory();
#else
  virtual AIDA::ITreeFactory* createTreeFactory(const std::string& = "");
  virtual AIDA::IHistogramFactory* createHistogramFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::IDataPointSetFactory* createDataPointSetFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::ITupleFactory* createTupleFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::IFunctionFactory* createFunctionFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::IFitFactory* createFitFactory(const std::string& = "");
  virtual AIDA::IPlotterFactory* createPlotterFactory
    (int = 0,char** = 0,const std::string& = "",const std::string& = "",const std::string& = "");

  virtual AIDA::IGenericFactory* createGenericFactory(const std::string& factoryType, const std::string& options = "");
  virtual AIDA::IGenericFactory* createManagedObjectGenericFactory(const std::string& factoryType,AIDA::ITree& tree,const std::string& options = "");

#endif
public: // Slash::Core::IManager :
  virtual std::string name() const;
  virtual void* cast(const std::string&) const;
public:
  AnalysisFactory(Slash::Core::ISession&);
  virtual ~AnalysisFactory();
protected:
  Slash::Core::ISession& fSession;
  Slash::Core::IWriter& fPrinter;
private:
  std::string fName;
  AIDA::IFunctionFactory* fFunctionFactory;
};

}

#endif
