//
// Embedding of the BatchLab/Helpers/AIDA.h file.
//

#ifndef BatchLab_Helpers_AIDA_h
#define BatchLab_Helpers_AIDA_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//  It relies on interfaces and c++ libraries only.
//

#include <AIDA/ITree.h>
#include <AIDA/ITreeFactory.h>
#include <AIDA/IManagedObject.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IHistogram3D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/IAnalysisFactory.h>
#include <AIDA/ICloud.h>
#include <AIDA/ICloud1D.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/ICloud3D.h>
#include <AIDA/IFunction.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IFunctionFactory.h>
#include <AIDA/IFunctionCatalog.h>
#include <AIDA/ITupleFactory.h>
#include <AIDA/ITuple.h>
#include <AIDA/IEvaluator.h>
#include <AIDA/IFilter.h>
#include <AIDA/IAxis.h>
#include <AIDA/IAnnotation.h>
#include <AIDA/IDataPointSetFactory.h>
#include <AIDA/IDataPointSet.h>
#include <AIDA/IFitFactory.h>
#include <AIDA/IFitter.h>
#include <AIDA/IPlotterFactory.h>
#include <AIDA/IPlotter.h>
#include <AIDA/IPlotterRegion.h>

#include <Lib/Manager.h> // Does not involve to link with Lib.
#include <Lib/Cast.h> // Does not involve to link with Lib.
#include <Lib/smanip.h>

#include <math.h> //sqrt

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// Not AIDA /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IAnalysisFactory* find_aida(
 Slash::Core::ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return  Lib_findManager(aSession,"AnalysisFactory",AIDA::IAnalysisFactory);
}
//////////////////////////////////////////////////////////////////////////////
inline std::string path_not_managed(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "/not_managed";   //Related to BatchLab::BaseFactory::manageObject.
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITree* find_tree(
 Slash::Core::ISession& aSession
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return Lib_findManager(aSession,aName,AIDA::ITree);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// Over AIDA only ///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ITree* find_memoryTree(
 AIDA::IAnalysisFactory& aAIDA
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITreeFactory* treeFactory = aAIDA.createTreeFactory();
  if(!treeFactory) return 0;
  AIDA::ITree* tree = treeFactory->create();
  delete treeFactory;
  return tree;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IBaseHistogram* find_baseHistogram(
 AIDA::ITree& aTree
,const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aPath);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IBaseHistogram);
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram1D* find_histogram1D(
 AIDA::ITree& aTree
,const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aPath);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IHistogram1D);
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IProfile1D* find_profile1D(
 AIDA::ITree& aTree
,const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aPath);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IProfile1D);
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram2D* find_histogram2D(
 AIDA::ITree& aTree
,const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aPath);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IHistogram2D);
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunction* find_function(
 AIDA::ITree& aTree
,const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aPath);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IFunction);
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ITuple* find_tuple(
 AIDA::ITree& aTree
,const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aPath);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::ITuple);
}
//////////////////////////////////////////////////////////////////////////////
inline std::string tuple_name(
 const AIDA::ITuple& aTuple
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const AIDA::IManagedObject* mobject = 
    Lib_ConstCast(aTuple,AIDA::IManagedObject);
  if(!mobject) return "";
  return mobject->name();
}
//////////////////////////////////////////////////////////////////////////////
inline std::string histogram_name(
 const AIDA::IBaseHistogram& aHistogram
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const AIDA::IManagedObject* mobject = 
    Lib_ConstCast(aHistogram,AIDA::IManagedObject);
  if(!mobject) return "";
  return mobject->name();
}
//////////////////////////////////////////////////////////////////////////////
/// Creators //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ITree* create_tree(
 AIDA::IAnalysisFactory& aAIDA
,const std::string& aStoreName
,const std::string& aStoreType
,bool aReadOnly
,bool aCreateNew
,const std::string& aOptions = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITreeFactory* treeFactory = aAIDA.createTreeFactory();
  if(!treeFactory) return 0;
  AIDA::ITree* tree = 
    treeFactory->create(aStoreName,aStoreType,aReadOnly,aCreateNew,aOptions);
  delete treeFactory;
  return tree;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram1D* create_histogram1D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram1D* histo = 
      factory->createHistogram1D(aPath,aTitle,aXnumber,aXmin,aXmax);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram1D* copy_histogram1D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::IHistogram1D& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram1D* histo = factory->createCopy(aPath,aHisto);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram2D* copy_histogram2D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::IHistogram2D& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram2D* histo = factory->createCopy(aPath,aHisto);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IBaseHistogram* copy_baseHistogram(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::IBaseHistogram& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  if(AIDA::IHistogram1D* object = Lib_Cast(aHisto,AIDA::IHistogram1D)) {
    AIDA::IHistogram1D* cobject = factory->createCopy(aPath,*object);
    delete factory;
    return cobject;
  } else if(AIDA::IHistogram2D* object = Lib_Cast(aHisto,AIDA::IHistogram2D)) {
    AIDA::IHistogram2D* cobject = factory->createCopy(aPath,*object);
    delete factory;
    return cobject;
  } else if(AIDA::IHistogram3D* object = Lib_Cast(aHisto,AIDA::IHistogram3D)) {
    AIDA::IHistogram3D* cobject = factory->createCopy(aPath,*object);
    delete factory;
    return cobject;
  } else {
    delete factory;
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram2D* create_histogram2D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram2D* histo = factory->createHistogram2D(aPath,aTitle,
    aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram3D* create_histogram3D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
,int aZnumber
,double aZmin
,double aZmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram3D* histo = factory->createHistogram3D(aPath,aTitle,
    aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax,aZnumber,aZmin,aZmax);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ICloud1D* create_cloud1D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,int aLimit = -1
,const std::string& aOptions = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::ICloud1D* cloud = factory->createCloud1D(aPath,aTitle,aLimit,aOptions);
  delete factory;
  return cloud;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ICloud2D* create_cloud2D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,int aLimit = -1
,const std::string& aOptions = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::ICloud2D* cloud = factory->createCloud2D(aPath,aTitle,aLimit,aOptions);
  delete factory;
  return cloud;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IDataPointSet* create_dataPointSet(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,int aDimension
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IDataPointSetFactory* factory = aAIDA.createDataPointSetFactory(aTree);
  if(!factory) return 0;
  AIDA::IDataPointSet* dps = factory->create(aPath,aTitle,aDimension);
  delete factory;
  return dps;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFitter* create_fitter(
 AIDA::IAnalysisFactory& aAIDA
,const std::string& aEngine = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFitFactory* factory = aAIDA.createFitFactory(); 
  if(!factory) return 0;
  AIDA::IFitter* fitter = factory->createFitter("",aEngine,"");
  delete factory;
  return fitter;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunction* create_function(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,const std::string& aModel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFunctionFactory* factory = aAIDA.createFunctionFactory(aTree); 
  if(!factory) return 0;
  AIDA::IFunction* function = factory->createFunctionByName(aPath,aModel);
  delete factory;
  if(function) function->setTitle(aTitle);
  return function;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunction* create_scripted_function(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,int aDimension
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFunctionFactory* factory = aAIDA.createFunctionFactory(aTree); 
  if(!factory) return 0;
  AIDA::IFunction* function = 
    factory->createFunctionFromScript(aPath,aDimension,aScript,"","","");
  delete factory;
  return function;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunctionCatalog* find_function_catalog(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFunctionFactory* factory = aAIDA.createFunctionFactory(aTree); 
  if(!factory) return 0;
  AIDA::IFunctionCatalog* catalog = factory->catalog();
  delete factory;
  return catalog;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ITuple* create_tuple(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,const std::string& aColumnNames
,const std::string& aOptions = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITupleFactory* factory = aAIDA.createTupleFactory(aTree);
  if(!factory) return 0;
  AIDA::ITuple* tuple = factory->create(aPath,aTitle,aColumnNames,aOptions);
  delete factory;
  return tuple;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IEvaluator* create_evaluator(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aExpression
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITupleFactory* factory = aAIDA.createTupleFactory(aTree);
  if(!factory) return 0;
  AIDA::IEvaluator* evaluator = factory->createEvaluator(aExpression);
  delete factory;
  return evaluator;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFilter* create_filter(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aExpression
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITupleFactory* factory = aAIDA.createTupleFactory(aTree);
  if(!factory) return 0;
  AIDA::IFilter* filter = factory->createFilter(aExpression);
  delete factory;
  return filter;
}
/*
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ITuple* copy_tuple(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::ITuple& aTuple
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITupleFactory* factory = aAIDA.createTupleFactory(aTree);
  if(!factory) return 0;
  AIDA::ITuple* tuple = factory->createCopy(aPath,aTuple); //Not yet in AIDA::ITuple
  delete factory;
  return tuple;
}
*/
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IPlotter* create_plotter(
 AIDA::IAnalysisFactory& aAIDA
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IPlotterFactory* factory = aAIDA.createPlotterFactory(); 
  if(!factory) return 0;
  AIDA::IPlotter* plotter = factory->create();
  delete factory;
  return plotter;
}
//////////////////////////////////////////////////////////////////////////////
inline bool region_parameterValue(
 AIDA::IPlotterRegion& aRegion
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s = aRegion.parameterValue(aWhat);
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline bool region_parameterValue(
 AIDA::IPlotterRegion& aRegion
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s = aRegion.parameterValue(aWhat);
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
/// Functions ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline bool is_Array2DFunction(
 const AIDA::IFunction& aFunction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::vector<double>& aparams = aFunction.parameters();

  // Analyse params to see if we have a Array2DFunction :
  bool status = false;
  if(aparams.size()>=3) {
    unsigned int dim = (unsigned int)aparams[0];
    if(dim==2) {
      unsigned int nx = (unsigned int)aparams[1];
      unsigned int ny = (unsigned int)aparams[2];
      if(1+1+1+nx+ny+nx*ny==aparams.size()) {
        status = true;
      }
    }
  }
  return status;
}
//////////////////////////////////////////////////////////////////////////////
inline bool is_ComplexArrayFunction(
 const AIDA::IFunction& aFunction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::vector<double>& aparams = aFunction.parameters();

  // Analyse params to see if we have a ComplexArrayFunction :
  bool status = false;
  if(aparams.size()>=2) {
    unsigned int dim = (unsigned int)aparams[0];
    if(dim==1) {
      unsigned int nx = (unsigned int)aparams[1];
      if(1+1+nx+2*nx==aparams.size()) {
        status = true;
      }
    }
  }

  return status;
}
//////////////////////////////////////////////////////////////////////////////
inline bool is_ArrayFunction(
 const AIDA::IFunction& aFunction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::vector<double>& aparams = aFunction.parameters();

  // Analyse params to see if we have a ArrayFunction :
  bool status = false;
  if(aparams.size()>=2) {
    unsigned int dim = (unsigned int)aparams[0];
    if(dim==1) {
      unsigned int nx = (unsigned int)aparams[1];
      if(1+1+nx+nx==aparams.size()) {
        status = true;
      }
    }
  }

  return status;
}
//////////////////////////////////////////////////////////////////////////////
/// Converters ///////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ICloud* convert_functionToCloud(
 AIDA::IHistogramFactory& aFactory
,const std::string& aPath
,const std::string& aTitle
,const AIDA::IFunction& aFunction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!is_Array2DFunction(aFunction)) return 0;

  AIDA::ICloud3D* cloud = aFactory.createCloud3D(aPath,aTitle); 
  if(!cloud) return 0;

  const std::vector<double>& aparams = aFunction.parameters();

  unsigned int nx = (unsigned int)aparams[1];
  unsigned int ny = (unsigned int)aparams[2];
  
  unsigned int index = 0;
  for(unsigned int j=0;j<ny;j++) {
    double y = aparams[3+nx+j];
    for(unsigned int i=0;i<nx;i++) {
      double x = aparams[3+i];
      double v = aparams[3+nx+ny+index];
      cloud->fill(x,y,v);
      index++;
    }
  }

  return cloud;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunction* convert_complexToRealFunction(
 AIDA::IFunctionFactory& aFactory
,const std::string& aPath
,const std::string& aTitle
,const AIDA::IFunction& aFunction
,const std::string& aWhat
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!is_ComplexArrayFunction(aFunction)) return 0;

  const std::vector<double>& aparams = aFunction.parameters();

  unsigned int number = (unsigned int)aparams[1];
  std::vector<double> params;
  params.push_back(1); //Dimension.
  params.push_back(number); //Number of entries for X axis.

  // Xs :
  unsigned int aparami = 2;
  unsigned int index;
  for(index=0;index<number;index++) {
    params.push_back(aparams[aparami]);
    aparami++;
  }
  // Values :
  if(aWhat=="imag") {
    for(unsigned int index=0;index<number;index++) {
      params.push_back(aparams[aparami+1]);
      aparami += 2;
    }
  } else if(aWhat=="norm") {
    for(unsigned int index=0;index<number;index++) {
      double real = aparams[aparami];
      double imag = aparams[aparami+1];
      params.push_back(real * real + imag * imag);
      aparami += 2;
    }
  } else if(aWhat=="abs") {
    for(unsigned int index=0;index<number;index++) {
      double real = aparams[aparami];
      double imag = aparams[aparami+1];
      params.push_back(::sqrt(real * real + imag * imag));
      aparami += 2;
    }
  } else { // real part :
    for(unsigned int index=0;index<number;index++) {
      params.push_back(aparams[aparami]);
      aparami += 2;
    }
  }

  AIDA::IFunction* function = aFactory.createFunctionByName(aPath,"Grid1D");
  if(function) {
    function->setTitle(aTitle);
    function->setParameters(params);
  }

  return function;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunction* convert_realToComplexFunction(
 AIDA::IFunctionFactory& aFactory
,const std::string& aPath
,const std::string& aTitle
,const AIDA::IFunction& aFunction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!is_ArrayFunction(aFunction)) return 0;

  const std::vector<double>& aparams = aFunction.parameters();

  unsigned int nx = (unsigned int)aparams[1];

  // ComplexGrid 1D function :
  std::vector<double> params;
  params.push_back(1); //Dimension.
  params.push_back(nx); //Number of entries for X axis.

  unsigned int aparami = 2;

  unsigned int index;
  for(index=0;index<nx;index++) {
    params.push_back(aparams[aparami]);
    aparami++;
  }
  for(index=0;index<nx;index++) {
    params.push_back(aparams[aparami]);
    aparami++;
    params.push_back(0);
  }

  AIDA::IFunction* function = 
    aFactory.createFunctionByName(aPath,"ComplexGrid1D");
  if(function) {
    function->setTitle(aTitle);
    function->setParameters(params);
  }

  return function;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ICloud* convert_tupleToCloud(
 AIDA::IHistogramFactory& aFactory
,const std::string& aPath
,const std::string& aTitle
,AIDA::ITuple& aTuple
,AIDA::IEvaluator& aEvaluatorX
,AIDA::IEvaluator& aEvaluatorY
,AIDA::IFilter& aFilter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ICloud2D* cloud = aFactory.createCloud2D(aPath,aTitle); 
  if(!cloud) return 0;

  if(!aEvaluatorX.initialize(aTuple)) return 0;
  if(!aEvaluatorY.initialize(aTuple)) return 0;
  if(!aFilter.initialize(aTuple)) return 0;

  aTuple.start();
  while(aTuple.next()) {
    if(!aFilter.accept()) continue;
    cloud->fill(aEvaluatorX.evaluateDouble(),aEvaluatorY.evaluateDouble());
  }

  return cloud;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunction* convert_tupleToFunction(
 AIDA::IFunctionFactory& aFactory
,const std::string& aPath
,const std::string& aTitle
,AIDA::ITuple& aTuple
,const std::string& aX
,AIDA::IEvaluator& aEvaluator
,AIDA::IFilter& aFilter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aEvaluator.initialize(aTuple)) return 0;
  if(!aFilter.initialize(aTuple)) return 0;

  int arrayX = aTuple.findColumn(aX);

  std::vector<double> Xs;
  std::vector<double> Vs;

  int NotFound = -1;

  // Loop on entries :
  aTuple.start();
  int row = 0;
  while(aTuple.next()) {

    if(arrayX!=NotFound) {
      Xs.push_back(aTuple.getDouble(arrayX));
    } else {
      Xs.push_back(row);
    }

    if(aFilter.accept()) {
      Vs.push_back(aEvaluator.evaluateDouble());
    } else {
      // Not taken.
      Vs.push_back(0);
    }

    row++;
  }

  // Grid 1D function :
  std::vector<double> params;
  params.push_back(1); //Dimension.
  params.push_back(row); //Number of entries for X axis.
  // Xs :
  int index;
  for(index=0;index<row;index++) params.push_back(Xs[index]);
  // Vs :
  for(index=0;index<row;index++) params.push_back(Vs[index]);

  AIDA::IFunction* function = aFactory.createFunctionByName(aPath,"Grid1D");
  if(function) {
    function->setTitle(aTitle);
    function->setParameters(params);
  }

  return function;
}

#endif
