@ECHO OFF
SETLOCAL

REM #///////////////////////////////////////////////////
REM #// If working with a osc_vis kit, customize the 
REM #// two below lines in order to find CLHEP and 
REM #// Geant4 on your machine. It is assumed that 
REM #// CLHEP installation is of the form :
REM #//   %CLHEP_home%\include
REM #//   %CLHEP_home%\lib\CLHEP-<version>.lib
REM #// and Geant4 :
REM #//   %g4_home%\include
REM #//   %g4_home%\lib\G4global.lib, etc..
REM #// It is assumed that the G4 global libs are
REM #// available.
REM #///////////////////////////////////////////////////
SET CLHEP_home=C:\cygwin\usr\local\CLHEP\2.0.3.1
SET g4_home=C:\cygwin\usr\local\geant4\8.2
REM #///////////////////////////////////////////
REM #///////////////////////////////////////////
REM #///////////////////////////////////////////


REM # Check that onx-config is available :
onx-config 1> NUL 2> NUL
IF ERRORLEVEL 1 (
  ECHO onx-config program not found.
  GOTO build_return
)

REM # Check that aida-config is available :
aida-config 1> NUL 2> NUL
IF ERRORLEVEL 1 (
  ECHO aida-config program not found.
  GOTO build_return
)

SET bin=.

REM # /GR to enable RTTI and dynamic cast.
REM # /GX to enable exception handling (for STL).
REM # /O2 optimize
REM # /W3 warmings
SET def_cppflags=/nologo /DWIN32 /MD /GX /GR /O2 /W3 /I. /I..

FOR /F "usebackq delims=;" %%s IN (`onx-config --cppflags`) DO SET onx_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --libs`) DO SET onx_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --iv_cppflags`) DO SET onx_iv_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --iv_libs`) DO SET onx_iv_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --core_libs`) DO SET onx_core_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`aida-config --cppflags`) DO SET aida_cppflags=%%s

REM # Check that g4lab-config is available and then 
REM # if we are under a osc_vis kit or a osc_g4 kit.
g4lab-config 1> NUL 2> NUL
IF ERRORLEVEL 1 GOTO end_if_not_g4lab_config

  REM ECHO g4lab-config found. Using a osc_g4 kit.

  REM # g4lab-config program found. We assume to use a osc_g4 kit.
  FOR /F "usebackq delims=;" %%s IN (`g4lab-config --cppflags`) DO SET g4lab_cppflags=%%s
  FOR /F "usebackq delims=;" %%s IN (`g4lab-config --libs`) DO SET g4lab_libs=%%s
  GOTO end_if_g4lab_config

:end_if_not_g4lab_config

  REM ECHO g4lab-config not found. Using a osc_vis kit.

  REM # g4lab-config program not found. We assume to use a osc_vis kit.
  REM # Geant4 and CLHEP specified in the g4_home and CLHEP_home
  REM # variables.
  REM # Try to find CLHEP and Geant4 at a specific place :
  IF EXIST %CLHEP_home% GOTO end_if_CLHEP_home
    ECHO CLHEP not found at %CLHEP_home%.
    ECHO Edit the build.bat script to set the CLHEP_home variable properly.
    GOTO build_return
:end_if_CLHEP_home
  IF EXIST %g4_home% GOTO end_if_g4_home
    ECHO Geant4 not found at %g4_home%.
    ECHO Edit the build script to set the g4_home variable properly.
    GOTO build_return
:end_if_g4_home

  SET g4_incs=/I%CLHEP_home%\include /I%g4_home%\include
  SET g4_libs=/LIBPATH:%g4_home%\lib libG4readout.a libG4run.a libG4event.a libG4tracking.a libG4processes.a libG4digits_hits.a libG4track.a libG4particles.a libG4geometry.a libG4graphics_reps.a libG4materials.a libG4intercoms.a libG4global.a /LIBPATH:%CLHEP_home%\lib CLHEP-2.0.3.1.lib
  SET g4lab_cppflags=/I..\source %g4_incs% %onx_cppflags%
  SET g4lab_libs=/LIBPATH:. G4LabUIOnX.lib G4LabCore.lib %g4_libs% %onx_core_libs%

  REM # Build G4LabCore and G4LabUIOnX libs :
  CALL build_G4Lab

:end_if_g4lab_config

REM #//////////////////////////////////////////////////
REM #// library with Geant4 user code :
REM #//////////////////////////////////////////////////
SET cppflags=%def_cppflags%
SET cppflags=%cppflags% %g4lab_cppflags%
cl.exe /c %cppflags% /Fo%bin%\CellHit.obj /Tp..\source\CellHit.cxx
cl.exe /c %cppflags% /Fo%bin%\CellSD.obj /Tp..\source\CellSD.cxx
cl.exe /c %cppflags% /Fo%bin%\Detector.obj /Tp..\source\Detector.cxx
cl.exe /c %cppflags% /Fo%bin%\PhysicsList.obj /Tp..\source\PhysicsList.cxx
cl.exe /c %cppflags% /Fo%bin%\Primary.obj /Tp..\source\Primary.cxx
cl.exe /c %cppflags% /Fo%bin%\EventAction.obj /Tp..\source\EventAction.cxx
cl.exe /c %cppflags% /Fo%bin%\RunAction.obj /Tp..\source\RunAction.cxx

SET G4_objs=%bin%\CellHit.obj %bin%\CellSD.obj %bin%\Detector.obj %bin%\PhysicsList.obj %bin%\Primary.obj %bin%\EventAction.obj  %bin%\RunAction.obj

REM #//////////////////////////////////////////////////
REM #// dll G4LabTemplateOne :
REM #//////////////////////////////////////////////////
SET cppflags=%def_cppflags%
SET cppflags=%cppflags% %onx_cppflags%
SET cppflags=%cppflags% %onx_iv_cppflags%
SET cppflags=%cppflags% %aida_cppflags%
SET cppflags=%cppflags% %g4lab_cppflags%
cl.exe /c %cppflags% /Fo%bin%\Callbacks.obj /Tp..\source\Callbacks.cxx
cl.exe /c %cppflags% /Fo%bin%\InventorCallbacks.obj /Tp..\source\InventorCallbacks.cxx

REM # Build the archive lib :
SET objs=%bin%\Callbacks.obj %bin%\InventorCallbacks.obj

lib.exe /nologo /machine:ix86 /out:%bin%\G4LabTemplateOne.arc %objs%

DEL %objs%

REM # Build the win32def.exe utility program :
cl.exe %def_cppflags% /Fe%bin%\win32def.exe /Tpwin32def.c

REM # Building the .def file :
dumpbin /out:%bin%\tmp /symbols %bin%\G4LabTemplateOne.arc
%bin%\win32def.exe G4LabTemplateOne < %bin%\tmp > %bin%\G4LabTemplateOne.def

REM # Build the DLL :
lib.exe /nologo /machine:ix86 /def:%bin%\G4LabTemplateOne.def /out:%bin%\G4LabTemplateOne.lib
IF EXIST %bin%\G4LabTemplateOne.dll DEL %bin%\G4LabTemplateOne.dll
SET libs=
SET libs=%libs% %onx_iv_libs%
SET libs=%libs% %onx_libs%
link.exe /nologo /dll /out:%bin%\G4LabTemplateOne.dll %bin%\G4LabTemplateOne.exp %bin%\G4LabTemplateOne.arc %libs%

REM #//////////////////////////////////////////////////
REM #// application G4LabTemplateOne_vis :
REM #//////////////////////////////////////////////////

REM # application G4LabTemplateOne_vis :
SET cppflags=%def_cppflags%
SET cppflags=%cppflags% %onx_cppflags%
SET cppflags=%cppflags% %onx_iv_cppflags%
SET cppflags=%cppflags% %aida_cppflags%
SET cppflags=%cppflags% %g4lab_cppflags%
cl.exe /c %cppflags% /Fo%bin%\G4LabTemplateOne_vis.obj /Tp..\applications\G4LabTemplateOne_vis.cxx
cl.exe /c %cppflags% /Fo%bin%\SteppingAction.obj /Tp..\applications\SteppingAction.cxx

SET objs=%bin%\G4LabTemplateOne_vis.obj %bin%\SteppingAction.obj %G4_objs%

IF EXIST %bin%\G4LabTemplateOne_vis.exe DEL %bin%\G4LabTemplateOne_vis.exe
SET libs=%onx_iv_libs%
SET libs=%libs% %onx_core_libs%
SET libs=%libs% %onx_libs%
SET libs=%libs% %g4lab_libs%
link.exe /nologo /out:%bin%\G4LabTemplateOne_vis.exe %objs% %libs%

DEL %bin%\tmp
DEL %objs%

:build_return

ENDLOCAL
@ECHO ON
