#ifndef Lib_Processor_h
#define Lib_Processor_h 

namespace Slash {namespace Core {class IWriter;}}
#include <string>
#include <vector>

#include <Lib/Value.h>
#include <Lib/Expression.h>
#include <Lib/ProcessorLexer.h>

namespace Lib {

class Processor {
public:
  Processor(const std::string&,Slash::Core::IWriter&);
  virtual ~Processor();
  Processor& operator=(const Processor&);
  void setScript(const std::string&);
  bool findVariables(std::vector<std::string>&);
  bool compile();
  bool compile(const std::vector<std::string>&);
  bool execute(const std::vector<Lib::Value>&,Lib::Value&,std::string&);
  bool cut(const std::vector<Lib::Value>&,bool&,std::string&);
  void clear();
public:  // Internal :
  bool findNames() const;
  void addName(const std::string&);
  bool findVariable(const std::string&,unsigned int&);
  const ProcessorLexer& lexer() const;
  Slash::Core::IWriter& printer() const;
  std::string error() const;
  std::vector<Expression*>& stack();
  void setResult(Expression*);
  void setStatus(bool);
private:
  std::string fString;
  Slash::Core::IWriter& fPrinter;
  bool fFindNames;
  std::vector<std::string> fNames;
  ProcessorLexer fLexer;
  Expression* fResult;
  bool fSuccess;
  std::vector<Expression*> fTmpExps;
  std::string fError;
};

}

#endif
