#ifndef OnX_WinUI_h
#define OnX_WinUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#include <windows.h>
#include <commctrl.h>

namespace WinTk {
  class Component;
  class MessageWindow;
  class ScrolledText;
  class MenuItem;
  class CallbackData;
  class Tree;
  class WarningDialog;
}

namespace OnX {class TermCyclic;}

namespace OnX {

class WinUI : public BaseUI {
public: //Slash::UI::IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();

  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool position(const std::string&,int&,int&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string& widget,
                     const std::string& file,
                     const std::string& format,
                     const std::string& options = "");
  virtual Slash::UI::ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Slash::Core::IMessage*);
  virtual std::string ask(const std::string&);
public: //OnX::BaseUI
  virtual int nativeSteer();
  virtual bool nativeSize(void*,unsigned int&,unsigned int&);
  virtual bool createNativeWidget(Lib::ItemML&,
                            const std::string&,
                            const std::string&,
                            Lib::ItemML*);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
  virtual bool enableTerminal(const std::string&,const std::string&);
  virtual bool disableTerminal();
public:
  WinUI(Slash::Core::ISession&,const std::string&,const std::vector<std::string>&);
  virtual ~WinUI();
  bool createTree(WinTk::Tree*,Lib::ItemML*);
  WinTk::Component* topWidget() const;
private:
  static void callback(WinTk::Component&,WinTk::CallbackData&,void*);
  static void destroyCallback(WinTk::Component&,WinTk::CallbackData&,void*);
  static void messageCallback(WinTk::Component&,WinTk::CallbackData&,void*);
  static void windowCallback(WinTk::Component&,WinTk::CallbackData&,void*);
  static LRESULT CALLBACK Win32WindowProc(HWND,UINT,WPARAM,LPARAM);
  //static std::string getXMLTree(HWND,HTREEITEM,int);
private:
  bool destroyOneWidget(Lib::ItemML&);
  int getType(const std::string&);
  void addCallbacks(Lib::ItemML&,WinTk::Component*,const std::string&);
  WinTk::MenuItem* findMenuItem(int);
protected:
  WinTk::Component* fTopWidget;
private:
  std::list<WinTk::Component*> fWidgets;
  // Console panel :
  WinTk::Component* fConsoleWindow;
  WinTk::ScrolledText* fConsoleText;
  int fActionID;
  WinTk::MessageWindow* fMessageWindow;
  WinTk::WarningDialog* fWarningDialog;
  std::list< std::pair<int,WinTk::MenuItem*> > fMenuItems;
  TermCyclic* fTermCyclic;
};

}

#endif

